/*---------------------------------------------------------------------------*\
 This file is part of the Fab@Home Project.
 Fab@Home operates under the BSD Open Source License.

 Copyright (c) 2009, Karl Gluck (kwg8@cornell.edu)
 
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
     * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.
     * Neither the name of the <organization> nor the
       names of its contributors may be used to endorse or promote products
       derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNERS OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
\*---------------------------------------------------------------------------*/
#pragma once

#include "slicematerial.h"

typedef QVector<SliceMaterial> SliceMaterialArray;

/**
 * Holds all of the information generated by the slicer for a cut through
 * a 3d model at some Z height over a plane parallel to XY at some Z
 * height.
 */
class Slice {
public:

  /**
   * Default constructor with an uninitialized Z-height and materials content
   */
  Slice();

  /**
   * Overloaded constructor for the z-coordinate of this slice
   *
   * @param Z the z-coordinate of this slice
   */
  Slice(float Z);

  /**
   * Overloaded constructor for the z-coordinate and materials of this slice
   *
   * @param Z the z-coordinate of this slice
   * @param materials the SliceMaterialArray that this slice contains
   */
  Slice(float Z,SliceMaterialArray materials);

  /** 
   * Returns all of the materials in this slice
   */
  const SliceMaterialArray& getMaterials() const;

  /** 
   * Returns the z-height of the slice
   */
  float getZ() const;

  /** 
   * Inserts all of the region information for this material type
   * into the slice.
   */
  void addSliceMaterial(SliceMaterial material);


private:

  /// All of the materials contained in this slice
  SliceMaterialArray materials;

  /// Height of this slice
  float Z;
};
